<?php
// vim: set ai ts=4 sw=4 ft=php:
//	License for all code of this FreePBX module can be found in the license file inside the module directory
//	Copyright 2014 Schmooze Com Inc.
//
namespace FreePBX\modules;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Process;
//progress bar
use Symfony\Component\Console\Helper\ProgressBar;
class Ucpnode extends \FreePBX_Helpers implements \BMO {
	private $nodever = "0.12.18";
	private $npmver = "2.15.11";
	private $icuver = "4.2.1";
	private $nodeloc = "/tmp";
	public function __construct($freepbx = null) {
		$this->db = $freepbx->Database;
		$this->freepbx = $freepbx;
		$this->nodeloc = __DIR__."/node";
	}

	public function install() {
		global $amp_conf;
		if(file_exists($amp_conf['AMPBIN']."/freepbx_engine_hook_ucpnode") && is_writable($amp_conf['AMPBIN']."/freepbx_engine_hook_ucpnode")) {
			unlink($amp_conf['AMPBIN']."/freepbx_engine_hook_ucpnode");
		}

		$output = exec("node --version"); //v0.10.29
		$output = str_replace("v","",trim($output));
		if(empty($output)) {
			out(_("Node is not installed"));
			return false;
		}
		if(version_compare($output,$this->nodever,"<")) {
			out(sprintf(_("Node version is: %s requirement is %s. Run 'yum upgrade nodejs' from the CLI as root"),$output,$this->nodever));
			return false;
		}


		$output = exec("npm --version"); //v0.10.29
		$output = trim($output);
		if(empty($output)) {
			out(_("Node Package Manager is not installed"));
			return false;
		}
		if(version_compare($output,$this->npmver,"<")) {
			out(sprintf(_("NPM version is: %s requirement is %s. Run 'yum upgrade nodejs' from the CLI as root"),$output,$this->npmver));
			return false;
		}

		$output = exec("icu-config --version"); //v4.2.1
		$output = trim($output);
		if(empty($output)) {
			out(_("icu is not installed. You need to run: yum install icu libicu-devel"));
			return false;
		}
		if(version_compare($output,$this->icuver,"<")) {
			out(sprintf(_("ICU version is: %s requirement is %s"),$output,$this->icuver));
			return false;
		}

		$webgroup = $this->freepbx->Config->get('AMPASTERISKWEBGROUP');

		$data = posix_getgrgid(filegroup($this->getHomeDir()));
		if($data['name'] != $webgroup) {
			out(sprintf(_("Home directory [%s] is not writable"),$this->getHomeDir()));
			return false;
		}

		if(file_exists($this->getHomeDir()."/.npm")) {
			$data = posix_getgrgid(filegroup($this->getHomeDir()."/.npm"));
			if($data['name'] != $webgroup) {
				out(sprintf(_("Home directory [%s] is not writable"),$this->getHomeDir()."/.npm"));
				return false;
			}
		}

		outn(_("Installing/Updating Required Libraries. This may take a while..."));
		if (php_sapi_name() == "cli") {
			out("The following messages are ONLY FOR DEBUGGING. Ignore anything that says 'WARN' or is just a warning");
		}

		$npmstatus = $this->pm2->installNodeDependencies($this->nodeloc,function($data) {
			outn($data);
		});
		if(!$npmstatus) {
			out("");
			out(_("Failed updating libraries!"));
		} else {
			out("");
			out(_("Finished updating libraries!"));
		}

		$set = array();
		$set['module'] = 'ucpnode';
		$set['category'] = 'UCP NodeJS Server';

		// NODEJSENABLED
		$set['value'] = true;
		$set['defaultval'] =& $set['value'];
		$set['options'] = '';
		$set['name'] = 'Enable the NodeJS Server';
		$set['description'] = 'Whether to enable the backend server for UCP which allows instantaneous updates to the interface';
		$set['emptyok'] = 0;
		$set['level'] = 1;
		$set['readonly'] = 0;
		$set['type'] = CONF_TYPE_BOOL;
		$this->freepbx->Config->define_conf_setting('NODEJSENABLED',$set);

		// NODEJSTLSENABLED
		$set['value'] = false;
		$set['defaultval'] =& $set['value'];
		$set['options'] = '';
		$set['name'] = 'Enable TLS for the NodeJS Server';
		$set['description'] = 'Whether to enable TLS for the backend server for UCP which allows instantaneous updates to the interface';
		$set['emptyok'] = 0;
		$set['level'] = 1;
		$set['readonly'] = 0;
		$set['type'] = CONF_TYPE_BOOL;
		$this->freepbx->Config->define_conf_setting('NODEJSTLSENABLED',$set);

		// NODEJSBINDADDRESS
		$set['value'] = '::';
		$set['defaultval'] =& $set['value'];
		$set['options'] = '';
		$set['name'] = 'NodeJS Bind Address';
		$set['description'] = 'Address to bind to. Default is "::" (Listen for all IPv4 and IPv6 Connections)';
		$set['emptyok'] = 0;
		$set['type'] = CONF_TYPE_TEXT;
		$set['level'] = 2;
		$set['readonly'] = 0;
		$this->freepbx->Config->define_conf_setting('NODEJSBINDADDRESS',$set);

		// NODEJSBINDPORT
		$set['value'] = '8001';
		$set['defaultval'] =& $set['value'];
		$set['options'] = '';
		$set['name'] = 'NodeJS Bind Port';
		$set['description'] = 'Port to bind to. Default is 8001';
		$set['emptyok'] = 0;
		$set['options'] = array(10,65536);
		$set['type'] = CONF_TYPE_INT;
		$set['level'] = 2;
		$set['readonly'] = 0;
		$this->freepbx->Config->define_conf_setting('NODEJSBINDPORT',$set);

		// NODEJSHTTPSBINDADDRESS
		$set['value'] = '::';
		$set['defaultval'] =& $set['value'];
		$set['options'] = '';
		$set['name'] = 'NodeJS HTTPS Bind Address';
		$set['description'] = 'Address to bind to. Default is "::" (Listen for all IPv4 and IPv6 Connections)';
		$set['emptyok'] = 0;
		$set['type'] = CONF_TYPE_TEXT;
		$set['level'] = 2;
		$set['readonly'] = 0;
		$this->freepbx->Config->define_conf_setting('NODEJSHTTPSBINDADDRESS',$set);

		// NODEJSHTTPSBINDPORT
		$set['value'] = '8003';
		$set['defaultval'] =& $set['value'];
		$set['options'] = '';
		$set['name'] = 'NodeJS HTTPS Bind Port';
		$set['description'] = 'Port to bind to. Default is 8003';
		$set['emptyok'] = 0;
		$set['options'] = array(10,65536);
		$set['type'] = CONF_TYPE_INT;
		$set['level'] = 2;
		$set['readonly'] = 0;
		$this->freepbx->Config->define_conf_setting('NODEJSHTTPSBINDPORT',$set);

		// NODEJSTLSCERTFILE
		$set['value'] = '';
		$set['defaultval'] =& $set['value'];
		$set['options'] = '';
		$set['name'] = 'NodeJS HTTPS TLS Certificate Location';
		$set['description'] = 'Sets the path to the HTTPS server certificate. This is required if "Enable TLS for the NodeJS Server" is set to yes.';
		$set['emptyok'] = 1;
		$set['type'] = CONF_TYPE_TEXT;
		$set['level'] = 2;
		$set['readonly'] = 0;
		$this->freepbx->Config->define_conf_setting('NODEJSTLSCERTFILE',$set);

		// NODEJSTLSPRIVATEKEY
		$set['value'] = '';
		$set['defaultval'] =& $set['value'];
		$set['options'] = '';
		$set['name'] = 'NodeJS HTTPS TLS Private Key Location';
		$set['description'] = 'Sets the path to the HTTPS private key. This is required if "Enable TLS for the NodeJS Server" is set to yes.';
		$set['emptyok'] = 1;
		$set['type'] = CONF_TYPE_TEXT;
		$set['level'] = 2;
		$set['readonly'] = 0;
		$this->freepbx->Config->define_conf_setting('NODEJSTLSPRIVATEKEY',$set);

		$this->freepbx->Config->commit_conf_settings();

		$cert = $this->freepbx->Certman->getDefaultCertDetails();
		if(!empty($cert)) {
			$this->setDefaultCert($cert, false);
		}

		if($this->freepbx->Modules->checkStatus("sysadmin")) {
			touch("/var/spool/asterisk/incron/ucpnode.logrotate");
		}

		//If we are root then start it as asterisk, otherwise we arent root so start it as the web user (all we can do really)
		outn(_("Stopping old running processes..."));
		$this->stopFreepbx();
		out(_("Done"));

		if($npmstatus) {
			outn(_("Starting new UCP Node Process..."));
			$started = $this->startFreepbx();
			if(!$started) {
				out(_("Failed!"));
			} else {
				out(sprintf(_("Started with PID %s!"),$started));
			}
		}
	}
	public function uninstall() {
		outn(_("Stopping old running processes..."));
		$this->stopFreepbx();
		out(_("Done"));
		exec("rm -Rf ".$this->nodeloc."/node_modules");
		try {
			$this->freepbx->Pm2->delete("ucpnode");
		} catch(\Exception $e) {}

	}
	public function backup(){

	}
	public function restore($backup){

	}

	public function setDefaultCert($details, $restart=true) {
		$certF = isset($details['integration']['files']['pem']) ? $details['integration']['files']['pem'] : $details['integration']['files']['crt'];
		$keyF = $details['integration']['files']['key'];
		$this->freepbx->Config->update("NODEJSTLSENABLED",true);
		$this->freepbx->Config->update("NODEJSTLSCERTFILE",$certF);
		$this->freepbx->Config->update("NODEJSTLSPRIVATEKEY",$keyF);
		if($restart) {
			try {
				$this->freepbx->Pm2->restart("ucpnode");
			} catch(\Exception $e) {}
		}
	}

	public function getHomeDir() {
		$webuser = \FreePBX::Freepbx_conf()->get('AMPASTERISKWEBUSER');
		$web = posix_getpwnam($webuser);
		$home = trim($web['dir']);
		if (!is_dir($home)) {
			// Well, that's handy. It doesn't exist. Let's use ASTSPOOLDIR instead, because
			// that should exist and be writable.
			$home = \FreePBX::Freepbx_conf()->get('ASTSPOOLDIR');
			if (!is_dir($home)) {
				// OK, I give up.
				throw new \Exception(sprintf(_("Asterisk home dir (%s) doesn't exist, and, ASTSPOOLDIR doesn't exist. Aborting"),$home));
			}
		}
		return $home;
	}

	public function dashboardService() {
		$service = array(
			'title' => _('UCP Daemon'),
			'type' => 'unknown',
			'tooltip' => _("Unknown"),
			'order' => 999,
			'glyph-class' => ''
		);
		$data = $this->freepbx->Pm2->getStatus("ucpnode");
		if(!empty($data) && $data['pm2_env']['status'] == 'online') {
			$uptime = $data['pm2_env']['created_at_human_diff'];
			$service = array_merge($service, $this->genAlertGlyphicon('ok', sprintf(_("Running (Uptime: %s)"),$uptime)));
		} else {
			$service = array_merge($service, $this->genAlertGlyphicon('critical', _("UCP Node is not running")));
		}

		return array($service);
	}

	/**
	 * Start FreePBX for fwconsole hook
	 * @param object $output The output object.
	 */
	public function startFreepbx($output=null) {
		if(!$this->freepbx->Config->get("NODEJSENABLED")) {
			return;
		}
		$status = $this->freepbx->Pm2->getStatus("ucpnode");
		switch($status['pm2_env']['status']) {
			case 'online':
				if(is_object($output)) {
					$output->writeln(sprintf(_("UCP Node Server has already been running on PID %s for %s"),$status['pid'],$status['pm2_env']['created_at_human_diff']));
				}
				return $status['pid'];
			break;
			default:
				if(is_object($output)) {
					$output->writeln(_("Starting UCP Node Server..."));
				}
				$this->freepbx->Pm2->start("ucpnode",__DIR__."/node/index.js");
				if(is_object($output)) {
					$progress = new ProgressBar($output, 0);
					$progress->setFormat('[%bar%] %elapsed%');
					$progress->start();
				}
				$i = 0;
				while($i < 100) {
					$data = $this->freepbx->Pm2->getStatus("ucpnode");
					if(!empty($data) && $data['pm2_env']['status'] == 'online') {
						if(is_object($output)) {
							$progress->finish();
						}
						break;
					}
					if(is_object($output)) {
						$progress->setProgress($i);
					}
					$i++;
					usleep(100000);
				}
				if(is_object($output)) {
					$output->writeln("");
				}
				if(!empty($data)) {
					$this->freepbx->Pm2->reset("ucpnode");
					if(is_object($output)) {
						$output->writeln(sprintf(_("Started UCP Node Server. PID is %s"),$data['pid']));
					}
					return $data['pid'];
				}
				if(is_object($output)) {
					$output->write("<error>".sprintf(_("Failed to run: '%s'")."</error>",$command));
				}
			break;
		}
		return false;
	}

	/**
	 * Stop FreePBX for fwconsole hook
	 * @param object $output The output object.
	 */
	public function stopFreepbx($output=null) {
		exec("pgrep -f ucpnode/node/node_modules/forever/bin/monitor",$o);
		if($o) {
			foreach($o as $z) {
				$z = trim($z);
				posix_kill($z, 9);
			}

			exec("pgrep -f ucpnode/node/index.js",$o);
			foreach($o as $z) {
				$z = trim($z);
				posix_kill($z, 9);
			}
		}

		$data = $this->freepbx->Pm2->getStatus("ucpnode");
		if(empty($data) || $data['pm2_env']['status'] != 'online') {
			if(is_object($output)) {
				$output->writeln("<error>"._("UCP Node Server is not running")."</error>");
			}
			return false;
		}

		// executes after the command finishes
		if(is_object($output)) {
			$output->writeln(_("Stopping UCP Node Server"));
		}

		$this->freepbx->Pm2->stop("ucpnode");

		$data = $this->freepbx->Pm2->getStatus("ucpnode");
		if (empty($data) || $data['pm2_env']['status'] != 'online') {
			if(is_object($output)) {
				$output->writeln(_("Stopped UCP Node Server"));
			}
		} else {
			if(is_object($output)) {
				$output->writeln("<error>".sprintf(_("UCP Node Server Failed: %s")."</error>",$process->getErrorOutput()));
			}
			return false;
		}

		return true;
	}

	/**
	 * FreePBX chown hooks
	 */
	public function chownFreepbx() {
		$files = array();
		return $files;
	}

	private function genAlertGlyphicon($res, $tt = null) {
		$glyphs = array(
			"ok" => "glyphicon-ok text-success",
			"warning" => "glyphicon-warning-sign text-warning",
			"error" => "glyphicon-remove text-danger",
			"unknown" => "glyphicon-question-sign text-info",
			"info" => "glyphicon-info-sign text-info",
			"critical" => "glyphicon-fire text-danger"
		);
		// Are we being asked for an alert we actually know about?
		if (!isset($glyphs[$res])) {
			return array('type' => 'unknown', "tooltip" => sprintf(_("Don't know what %s is").$res), "glyph-class" => $glyphs['unknown']);
		}

		if ($tt === null) {
			// No Tooltip
			return array('type' => $res, "tooltip" => null, "glyph-class" => $glyphs[$res]);
		} else {
			// Generate a tooltip
			$html = '';
			if (is_array($tt)) {
				foreach ($tt as $line) {
					$html .= htmlentities($line, ENT_QUOTES, "UTF-8")."\n";
				}
			} else {
				$html .= htmlentities($tt, ENT_QUOTES, "UTF-8");
			}

			return array('type' => $res, "tooltip" => $html, "glyph-class" => $glyphs[$res]);
		}
		return '';
	}

	private function generateRunAsAsteriskCommand($command) {
		return $this->freepbx->Pm2->generateRunAsAsteriskCommand($command,$this->nodeloc);
	}
}
